##demo for longest common subsequence

library(qualV)
library(ggplot2)

setwd("/Users/qhe/Documents/Britt temp/IMPSworkshopCodes")

##read in  demo_data and example predefined reference sequence
DATA <- read.csv(file="data_demo.csv",header = TRUE,sep=",")
RS <- read.csv(file="RefSeq.csv",header = TRUE,sep=",")

##===================================================
##longest common subsequence example by one person 
##===================================================

num <-1 ##pick a number 1-600

obs <- as.character(DATA$Coded_Action_Sequences[num])
obs_seq <- unlist(strsplit(obs, ", "))


result<-{}
for (j in 1:nrow(RS)){
  
  ref <- as.character(RS$RefSeq[j])  ##reference:predefined action sequence
  ref_seq <- unlist(strsplit(ref,", "))
  A <- LCS(obs_seq,ref_seq)
  obs <- paste(A$a,collapse=",")     ##observation action sequence
  Lobs <- as.numeric(length(A$a))    ##length of observation sequence
  ref <- paste(A$b,collapse=",")     ##reference action sequence (predefined)
  Lref <- as.numeric(length(A$b))    ##length of reference sequence
  LCS <- paste(A$LCS,collapse=",")   ##longest common subsequence
  LLCS <- as.numeric(as.character(A$LLCS))      ##length of longest common subsequence between observation and reference
  eff <- round(LLCS/Lobs,3)                     ##efficiency. length of LCS/length of obersvation
  sim <- round (LLCS/Lref,3)                    ##similarity. length of LCS/length of references
  path=j  ##path number
  
  LCSresult<- cbind(obs, Lobs, ref, Lref, LCS, LLCS, eff, sim, path)
  result <- as.data.frame(rbind(result,LCSresult))
  
}

##===========================================================================================================
##loop for getting longest common subsequence for all people (output the maximum similarity for each person)
##person i by predefined sequence j
##===========================================================================================================
output <- {}
for (i in 1:nrow(DATA)) {
  obs <- as.character(DATA$Coded_Action_Sequences[i])
  obs_seq <- unlist(strsplit(obs, ", "))
  
  result<-{}
  for (j in 1:nrow(RS)){
    
    ref <- as.character(RS$RefSeq[j])  ##reference:predefined action sequence 1:j
    ref_seq <- unlist(strsplit(ref,", "))
    A <- LCS(obs_seq,ref_seq)        ##calculate longest common subsequence
    obs <- paste(A$a,collapse=",")   ##observation action sequence
    Lobs <- as.numeric(length(A$a))  ##length of observation sequence
    ref <- paste(A$b,collapse=",")   ##reference action sequence (predefined)
    Lref <- as.numeric(length(A$b))  ##length of reference sequence
    LCS <- paste(A$LCS,collapse=",") ##longest common subsequence
    LLCS <- as.numeric(as.character(A$LLCS)) ##length of longest common subsequence between observation and reference
    eff <- round(LLCS/Lobs,3)                ##efficiency. length of LCS/length of obersed sequence, to show how much redundant actions
    sim <- round (LLCS/Lref,3)               ##similarity. length of LCS/length of reference sequence, to show how much the individual's sequence is similar to the predefined one.
    path=j  ##path number
    
    LCSresult<- cbind(obs, Lobs, ref, Lref, LCS, LLCS, eff, sim, path)
    result <- as.data.frame(rbind(result,LCSresult))
  }
  
  result$sim <- as.numeric(levels(result$sim)[result$sim]) ##convert factor to numberic
  result$eff <- as.numeric(levels(result$eff)[result$eff]) ##convert factor to numberic
  sorted <- result[order(result$sim,decreasing = TRUE),]   ##sorting by sim
  max <- sorted[1,]  ##extract the first line from sorting as the highest similarity
  output <- rbind(output,max)
    
}

for (i in 1:nrow(output)){ 
  if (output$obs[i]=="Start,Next,Next_OK"){
    output$sim[i] <- NA
    output$eff[i] <- NA
    output$path[i] <- NA
  }
}

##combine with input dataset

comDATA <- cbind.data.frame(DATA,output)

##distribution of similarity in correct/incorrect group
correct <- subset(comDATA,score=="1")
incorrect <- subset(comDATA,score=="0")
summary (as.numeric(correct$sim))
summary (as.numeric(incorrect$sim))

summary (as.numeric(correct$eff))
summary (as.numeric(incorrect$eff))

simplot <- ggplot(na.omit(comDATA), aes(x=as.character(score), y=sim, color=score)) +
  geom_boxplot()
simplot+
  facet_wrap(~Country)


effplot <- ggplot(na.omit(comDATA), aes(x=as.character(score), y=eff, color=score)) +
  geom_boxplot()
effplot+
  facet_wrap(~Country)

