##demo for n-grams

library (ngram)

setwd("/Users/qhe/Documents/Britt temp/IMPSworkshopCodes")

##read in  demo_data
DATA <- read.csv(file="data_demo.csv",header = TRUE,sep=",")

##===================================================
##n-grams model for one sequence example
##===================================================
num <- 1 ##set a number between 1-600
obs <- as.character(DATA$Coded_Action_Sequences[num])
obs <- gsub(',','',obs)  ##remove the comma between each action

##ngrams
ng1<-ngram(obs,n=1) ##unigram
ng1

ng2<-ngram(obs,n=2) ##bigram
ng2

ng3<-ngram(obs,n=3) ##trigram
ng3

##summarize n-grams
get.phrasetable(ng1)
get.phrasetable(ng2)
get.phrasetable(ng3)


##==================================================
##n-grams in whole data corpus
##==================================================
N <- 200 ##use the first 200 respondents as an example, focus on country 1 sample only.

ActSeq<-as.character(DATA$Coded_Action_Sequences)
ActSeqList<-list()
for(i in 1:N){
  ActSeqList[[i]]<-unlist(strsplit(ActSeq[i], split=", "))[1:length(unlist(strsplit(ActSeq[i], split=", ")))]
}

sort(unique(unlist(ActSeqList)))
table(unlist(ActSeqList))


# action sequence as one string per person
ActionSequences<-lapply(ActSeqList,concatenate)

#======================================================================
# analysis unigrams and bigrams
#=======================================================================
l2<-unlist(lapply(ActionSequences, function(x) length(unlist(strsplit(x, split=" ")))>3)) # extract all action sequences >3 (exclude people with "Start, Next, Next_OK")
score<-DATA$score[which(l2 == TRUE)] # scores of action sequences >3


##uniqrams analysis
ng1<-lapply(ActionSequences[which(l2 == TRUE)], function(x) ngram(x,n=1)) # extract all unigrams (from non-skipping respondents)
ngs1<-unlist(lapply(ng1, function(x) get.ngrams(x))) ##all unigram token (with repetition)
uni_type <- unique(ngs1) ## all unigram type (without repetition)
length_uni_type <-length(unique(ngs1)) ## how many unique uniqrams used by the sample
tail(sort(table(ngs1)))
Nng1<-length(ng1) # number of effective sequences (exclude skipping) equals to number of respondents who should be taken into account 


##bigrams analysis
ng2<-lapply(ActionSequences[which(l2 == TRUE)], function(x) ngram(x,n=2)) # extract all bigrams (from non-skipping respondents)
ngs2<-unlist(lapply(ng2, function(x) get.ngrams(x))) ##all bigrams token (with repetition)
bi_type <- unique (ngs2) ## all bigram type (without repetion)
length_bi_type <- length(unique(ngs2)) ## how many unique bigrams used by the sample
tail(sort(table(ngs2)))
Nng2<-length(ng2) # number of effective sequences (exclude skipping)

# Results table: frequencies and weighted frequencies
wgtng2<-setNames(data.frame(matrix(ncol =4, nrow =Nng2*length(unique(ngs2)))), c("ngram","freq","weightfreq", "score")) 
wgtng2[,1]<-rep(unique(ngs2),Nng2)
for(i in 1:length(unique(ngs2))){
  ISF<-log(Nng2/sum(unlist(lapply(ng2, function(x) unique(ngs2)[i]%in%get.ngrams(x))))) # inverse sequence frequency of bigram
  print(ISF)
  for(n in 1:Nng2){
    nG<-get.ngrams(ng2[[n]]) # bigrams for person
    if(unique(ngs2)[i] %in% nG){
      freq<-sum(unique(ngs2)[i]==nG) # frequnecy in person-level sequeence
      weightfreq<-freq*(1+log(freq))*ISF # weighted frequency person-level sequeence
      wgtng2[(n-1)*length(unique(ngs2))+i,2:4]<-c(freq,weightfreq,score[n]) # fill table
    }else{
      wgtng2[(n-1)*length(unique(ngs2))+i,2:4]<-c(0,0,score[n]) # set frequencies to zero if bigram did not occur in person-specific sequence
    }
  }
}

# sum of frequencies and weighted frequencies by correct and incorrect group 
## the number of Freqs_re_reliable is twice as action type, because of the (weighted) frequence for each bigram are different in correct and incorrect groups are different
Freqs<-aggregate(wgtng2$weightfreq, by=list(wgtng2$ngram,wgtng2$score), sum)
names(Freqs)<-c("ngram","score","weightfreq")
Freqs$freqs<-aggregate(wgtng2$freq, by=list(wgtng2$ngram,wgtng2$score), sum)[3]
names(Freqs)[4] <- c("freq")



# contingency table and chi2
CT<-list()
TabChsqr<-setNames(data.frame(matrix(ncol =3, nrow =length(unique(Freqs$ngram)))), c("ngram","chsiqr","ratio")) 
for(i in 1:length(unique(Freqs$ngram))){
  # contingency table
  CT[[i]]<-matrix(NA,nrow=2,ncol=2)
  CT[[i]][1,1]<-Freqs$weightfreq[Freqs$ngram==unique(Freqs$ngram)[i]&Freqs$score==1] # occurrence correct
  CT[[i]][1,2]<-Freqs$weightfreq[Freqs$ngram==unique(Freqs$ngram)[i]&Freqs$score==0] # occurrence incorrect
  CT[[i]][2,1]<-sum(Freqs$weightfreq[Freqs$score==1])-CT[[i]][1,1] # nonoccurrence correct
  CT[[i]][2,2]<-sum(Freqs$weightfreq[Freqs$score==0])-CT[[i]][1,2] # nonoccurrence incorrect
  TabChsqr[i,1]<-unique(Freqs$ngram)[i] # bigram
  TabChsqr[i,2]<-chisq.test(CT[[i]])$statistic # chi square
  TabChsqr[i,3]<-ifelse(CT[[i]][1,1]/CT[[i]][1,2]>sum(Freqs$weightfreq[Freqs$score==1])/sum(Freqs$weightfreq[Freqs$score==0]),1,0) # calculate ratio
}


TabChsqr<-TabChsqr[TabChsqr$chsiqr>3.84,] # eliminate sequences with small chisqr
TabChsqr<-TabChsqr[order(TabChsqr$ratio,TabChsqr$chsiqr, decreasing = TRUE),] 
TabChsqr_correct <- subset(TabChsqr,ratio==1)
TabChsqr_incorrect<- subset(TabChsqr,ratio==0)

  
  

